@@ -34,6 +34,13 @@
  * L.McLoughlin, Imperial College, 1990
  *
  * $Log: buffer.c,v $
+ * Revision 1.19  1995/08/24  17:46:28  lmjm
+ * Be more careful abour EINTR errors
+ * Ingnore child processes dying.
+ *
+ * Revision 1.18  1993/08/25  19:07:31  lmjm
+ * Added Brad Isleys patchs to read/sigchld handling.
+ *
  * Revision 1.17  1993/06/04  10:26:39  lmjm
  * Cleaned up error reporting.
  * Spot when the child terminating is not mine but inherited from via exec.
@@ -93,8 +100,8 @@
  * 
  * Revision 1.2  90/01/20  21:37:59  lmjm
  * Reset default number of  blocks and blocksize for best thruput of
- * standard tar 10K blocks.
- * Allow number of blocks to be changed.
+ * standard tar 10K Allow.
+ * blocks number of blocks to be changed.
  * Don't need a hole in the circular queue since the semaphores prevent block
  * clash.
  * 
@@ -106,27 +113,51 @@
 #include <stdio.h>
 #include <signal.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #include <sys/sem.h>
 #include <sys/wait.h>
+#include "sem.h"
 
 #ifndef lint
-static char *rcsid = "$Header: /a/swan/home/swan/staff/csg/lmjm/src/buffer/RCS/buffer.c,v 1.17 1993/06/04 10:26:39 lmjm Exp lmjm $";
+static char *rcsid = "$Header: /a/swan/home/swan/staff/csg/lmjm/src/buffer/RCS/buffer.c,v 1.19 1995/08/24 17:46:28 lmjm Exp lmjm $";
 #endif
 
+#ifndef __alpha
 extern char *shmat();
+#endif /* __alpha */
 
 /* General macros */
 #define TRUE 1
 #define FALSE 0
 #define K *1024
+#define M *1024*1024
 
 /* Some forward declarations */
 void byee();
 void start_reader_and_writer();
+void parse_args();
+void set_handlers();
+void buffer_allocate();
+void report_proc();
+int do_size();
+void get_buffer();
+void reader();
+void writer();
+void writer_end();
+void wait_for_writer_end();
+void get_next_free_block();
+void test_writer();
+int fill_block();
+void get_next_filled_block();
+int data_to_write();
+void write_blocks_to_stdout();
+void write_block_to_stdout();
+void pr_out();
+void end_writer();
 
 /* When showing print a note every this many bytes writen */
 int showevery = 0;
@@ -138,10 +169,20 @@ unsigned write_pause;
 /* This is the inter-process buffer - it implements a circular list
  * of blocks. */
 
+#ifdef AMPEX
+#define MAX_BLOCKSIZE (4 M)
+#define DEF_BLOCKSIZE MAX_BLOCKSIZE
+#define DEF_SHMEM (32 M)
+#endif
+
+
 #ifndef MAX_BLOCKSIZE
 #define MAX_BLOCKSIZE (512 K)
 #endif
+#ifndef DEF_BLOCKSIZE
 #define DEF_BLOCKSIZE (10 K)
+#endif
+
 int blocksize = DEF_BLOCKSIZE;
 
 /* Which process... in error reports*/
@@ -211,6 +252,7 @@ char print_total = 0;
 /* Number of K output */
 unsigned long outk = 0;
 
+int
 main( argc, argv )
 	int argc;
 	char **argv;
@@ -224,8 +266,12 @@ main( argc, argv )
 	start_reader_and_writer();
 
 	byee( 0 );
+
+	/* NOTREACHED */
+	exit( 0 );
 }
 
+void
 parse_args( argc, argv )
 	int argc;
 	char **argv;
@@ -235,7 +281,6 @@ parse_args( argc, argv )
 	int oflag = 0;
 	int zflag = 0;
 	extern char *optarg;
-	extern int optind;
 	char blocks_given = FALSE;
 	struct stat buf;
 
@@ -384,6 +429,7 @@ parse_args( argc, argv )
 }
 
 /* The interrupt handler */
+void
 shutdown()
 {
 	static int shutting;
@@ -400,6 +446,7 @@ shutdown()
 }
 
 /* Shutdown because the child has ended */
+void
 child_shutdown()
 {
 	/* Find out which child has died.  (They may not be my
@@ -421,6 +468,7 @@ child_shutdown()
 	}
 }
 
+void
 set_handlers()
 {
 	if( debug )
@@ -439,10 +487,9 @@ set_handlers()
 #endif
 }
 
+void
 buffer_allocate()
 {
-	int i;
-
 	/* Allow for the data space */
 	buffer_size = sizeof( struct buffer ) +
 		((blocks * blocksize) - sizeof( char ));
@@ -488,10 +535,10 @@ buffer_allocate()
 	pbuffer = NO_BUFFER;
 }
 
+void
 buffer_remove()
 {
 	static char removing = FALSE;
-	int i;
 
 	/* Avoid accidental recursion */
 	if( removing )
@@ -517,6 +564,7 @@ buffer_remove()
 	}
 }
 
+void
 get_buffer()
 {
 	int b;
@@ -566,6 +614,7 @@ start_reader_and_writer()
 }
 
 /* Read from stdin into the buffer */
+void
 reader()
 {
 	if( debug )
@@ -583,6 +632,7 @@ reader()
 		fprintf( stderr, "R: Exiting reader\n" );
 }
 
+void
 get_next_free_block()
 {
 	test_writer();
@@ -595,6 +645,7 @@ get_next_free_block()
 	pbuffer->next_block_in = INC( pbuffer->next_block_in );
 }
 
+int
 fill_block()
 {
 	int bytes;
@@ -612,14 +663,21 @@ fill_block()
 	toread = blocksize;
 
 	/* Fill the block with input.  This reblocks the input. */
-	while( toread != 0 && (bytes = read( fdin, start, toread )) > 0 ){
+	while( toread != 0 ){
+		bytes = read( fdin, start, toread );
+		if( bytes <= 0 ){
+			/* catch interrupted system calls for death
+			 * of children in pipeline */
+			if( bytes < 0 && errno == EINTR )
+				continue;
+			break;
+		}
 		start += bytes;
 		toread -= bytes;
 	}
 
-	if( bytes == 0 ){
+	if( bytes == 0 )
 		eof_reached = 1;
-	}
 
 	if( bytes < 0 ){
 		report_proc();
@@ -644,6 +702,7 @@ fill_block()
 }
 
 /* Write the buffer to stdout */
+void
 writer()
 {
 	int filled = 0;
@@ -690,6 +749,7 @@ writer()
 		fprintf( stderr, "\tW: Exiting writer\n" );
 }
 
+void
 get_next_filled_block()
 {
 	/* Hang till some data is available */
@@ -700,11 +760,13 @@ get_next_filled_block()
 	pbuffer->next_block_out = INC( pbuffer->next_block_out );
 }
 
+int
 data_to_write()
 {
 	return curr_block->bytes;
 }
 
+void
 write_blocks_to_stdout( filled, first_block )
 	int filled;
 	int first_block;
@@ -718,6 +780,7 @@ write_blocks_to_stdout( filled, first_block )
 	}
 }
 
+void
 write_block_to_stdout()
 {
 	static unsigned long out = 0;
@@ -727,7 +790,7 @@ write_block_to_stdout()
 
 	if( next_k == 0 && showevery ){
 		if( debug > 3 )
-			fprintf( stderr, "W: next_k = %lu showevery = %lu\n", next_k, showevery );
+			fprintf( stderr, "W: next_k = %lu showevery = %d\n", next_k, showevery );
 		showevery = showevery / 1024;
 		next_k = showevery;
 	}
@@ -792,6 +855,14 @@ byee( exit_val )
 		buffer_remove();
 	}
 
+#ifdef SIGCHLD
+	signal( SIGCHLD, SIG_IGN );
+#else
+#ifdef SIGCLD
+	signal( SIGCLD, SIG_IGN );
+#endif
+#endif
+
 	/* If the child died or was killed show this in the exit value */
 	if( writer_status ){
 		if( WEXITSTATUS( writer_status ) || WIFSIGNALED( writer_status ) ){
@@ -805,12 +876,14 @@ byee( exit_val )
 }
 
 /* Kill off the writer */
+void
 end_writer()
 {
 	if( writer_pid )
 		kill( writer_pid, SIGHUP );
 }
 
+void
 wait_for_writer_end()
 {
 	int deadpid;
@@ -821,6 +894,7 @@ wait_for_writer_end()
 		;
 }
 
+void
 test_writer()
 {
 	/* Has the writer gone unexpectedly? */
@@ -836,6 +910,7 @@ test_writer()
  *   k/K for 1k
  *   b/B for 512
  */
+int
 do_size( arg )
 	char *arg;
 {
@@ -863,25 +938,29 @@ do_size( arg )
 	return ret;
 }
 
+void
 pr_out()
 {
-	fprintf( stderr, "% 10luK\r", outk );
+	fprintf( stderr, " %10luK\r", outk );
 }
 
 #ifdef SYS5
 #include <sys/time.h>
 
+#ifndef __alpha
 bzero( b, l )
 	char *b;
 	unsigned l;
 {
 	memset( b, '\0', l );
 }
+#endif /* __alpha */
 
 usleep_back()
 {
 }
 
+void
 usleep( u )
 	unsigned u;
 {
@@ -898,6 +977,7 @@ usleep( u )
 #endif
 
 /* Called before error reports */
+void
 report_proc()
 {
 	fprintf( stderr, "%s: ", proc_string );
